
// Revised
/* ++++++++++

   FILE:  TarNgzip.cpp
   REVS:  $Revision: b1.03 $
   NAME:  Bryan Becker
   DATE:  Fri May 10, 1996

    Modifications Geek House Software freeware.
    -------------------------------------------
	 b1.03 
	 - many optimizations could be made (re-using variables, piping from
	   tar to gzip, subroutine) but this version is easy to read. I will post this in
	   the next couple of days along with some other utils.

+++++ */

// Original Code:
/* ++++++++++

   FILE:  make_tar.cpp
   REVS:  $Revision: 1.1 $
   NAME:  steve
   DATE:  Fri Jan 26 16:09:34 PST 1996

   Copyright (c) 1996 by Be Incorporated.  All Rights Reserved.

+++++ */

#include <Directory.h>
#include <Window.h>
#include <Rect.h>
#include <View.h>
#include <File.h>
#include <Alert.h>
#include <AppDefs.h>
#include <Message.h>
#include <Errors.h>

#include <Debug.h>

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>


extern char** environ;
static void DbugMsg(char *psMsg);

#define TAR_PARAMS 3
#define GZIP_PARAMS 3

const char TAR_FILE_EXT[] = {"tar"};
const char GZIP_FILE_EXT[] = {"gzip"};

class TStatusView : public BView {
public:
		char*		drawstring;		
					TStatusView(BRect,char *winstr);
virtual void		Draw(BRect);
};

class TStatusWindow : public BWindow {
public:
					TStatusWindow(char *winstr);
};

#pragma export on
void process_refs(record_ref dir_ref, BMessage* msg, void*);
#pragma export reset

void process_refs(record_ref dir_ref, BMessage* msg, void*)
{

	BAlert*			alert;
	char**			t_argv;
	char**			g_argv;
	char			name[B_FILE_NAME_LENGTH];
	char			archive_name[B_FILE_NAME_LENGTH];
	char 			basic_arc_name[B_FILE_NAME_LENGTH];
	char 			gzip_name[B_FILE_NAME_LENGTH];
	
	TStatusWindow*	wind;
	ulong			type;
	record_ref		ref;
	long			result;
	thread_info		info;
	long			count;
	long			i;
	BFile			file;
	BStore*			store_ptr;
	BDirectory		dir;
	char			path[1000];
	thread_id		tid;
	thread_id		gtid;
	BFile 			TarTargetF;
	BDirectory 		TarTargetD;
		
	
	//-----------------------------------------------------------------	
	// this add-on requires a selection
	//-----------------------------------------------------------------	

	msg->GetInfo("refs", &type, &count);
	if (count == 0) {
		alert = new BAlert("", "You must have a selection to make a tar archive.", "Cancel");
		alert->Go();
		return;
	}

	dir.SetRef(dir_ref);
	if (dir.Error() != B_NO_ERROR) {
		alert = new BAlert("", "You must make tar archives from folders.", "Cancel");
		alert->Go();
		return;
	}

	dir.GetPath(path, 1000);
	chdir(path);
	
	
	//-----------------------------------------------------------------	
	//
	// create an original name for archive
	// 
	//-----------------------------------------------------------------	

	ref = msg->FindRef("refs", 0);
	if (does_ref_conform(ref, "File")) 
		{
		TarTargetF.SetRef(ref);
		store_ptr = &TarTargetF;
		} 
	else 
		{
		TarTargetD.SetRef(ref);
		store_ptr = &TarTargetD;

		}
		
		
	store_ptr->GetName(name);
	strcpy(archive_name, name);	
	strcat(archive_name, ".tar");
	
	strcpy(gzip_name, archive_name);
	strcat(gzip_name, ".gz");
	
	// store the basic arc name 
	strcpy(basic_arc_name, name);	
		
	i = 2;

	//-----------------------------------------------------------------	
	// Check the name of the new archive file -- if it exists insert an
	// integer before the extension and loop until a unique name is found
	//-----------------------------------------------------------------	
	while ((dir.Contains(archive_name)) || (dir.Contains(gzip_name)) )
	{
	
		sprintf(archive_name, "%s_%d.%s", basic_arc_name, i++, TAR_FILE_EXT);
		strcpy(gzip_name, archive_name);
		strcat(gzip_name, ".gz");
	}
	
	//-----------------------------------------------------------------	
	//
	// THE TAR SECTION
	//
	//-----------------------------------------------------------------	

	//-----------------------------------------------------------------	
	//
	// create argv style buffer to pass to tar tool
	// need string for each entry, default params, and null entry at end
	//
	//-----------------------------------------------------------------	
	t_argv = (char**)malloc((count + TAR_PARAMS + 1) * (sizeof(char*)));

	for (i = 0; i < count; i++) {
		ref = msg->FindRef("refs", i);
		if (does_ref_conform(ref, "File")) {
			file.SetRef(ref);
			store_ptr = &file;
		} else {
			dir.SetRef(ref);
			store_ptr = &dir;
		}
			
		if (store_ptr->Error() != B_NO_ERROR) {
			alert = new BAlert("", "Sorry, you cannot archive database-only items.", "Cancel");
			alert->Go();
			count = i;
			goto exit;
		}
			
		store_ptr->GetName(name);
		t_argv[i + TAR_PARAMS] = (char*)malloc(strlen(name) + 1);
		strcpy(t_argv[i + TAR_PARAMS], name);
	}

	//-----------------------------------------------------------------	
	//
	// launch tar tool with argv buffer
	//
	//-----------------------------------------------------------------	

	t_argv[0] = "/boot/bin/tar";
	t_argv[1] = "cf";
	t_argv[2] = archive_name;
	t_argv[i + TAR_PARAMS] = 0;
	tid = load_posix(t_argv[0], t_argv, environ);
	if (tid > 0) {
		resume_thread(tid);
	
		// wait a little bit before checking thread status
		snooze(300000.0);

		// if the thread is still around then bring up status window
		if (get_thread_info(tid, &info) == B_NO_ERROR) {
			wind = new TStatusWindow("Creating tar file...");
			wait_for_thread(tid, &result);
			wind->Close();
		}
	} else {
		alert = new BAlert("", "Sorry, could not find the tar tool to make an archive.", "Cancel");
		alert->Go();
	}
	

	//-----------------------------------------------------------------	
	// 
	// THE GZIP SECTION
	// 
	//-----------------------------------------------------------------	

	//-----------------------------------------------------------------	
	// create argv style buffer to pass to gzip
	// need string for each entry, default params, and null entry at end
	//-----------------------------------------------------------------
	g_argv = (char**)malloc(GZIP_PARAMS * (sizeof(char*)));
		
	g_argv[0] = "/boot/bin/gzip";
	g_argv[1] = archive_name;
	g_argv[2] = 0;
	
	chdir(path);
	//-----------------------------------------------------------------	
	// launch gzip tool with argv buffer
	//-----------------------------------------------------------------		
	gtid = load_posix(g_argv[0], g_argv, environ);
	if (gtid > 0) {
		resume_thread(gtid);
	
		// wait a little bit before checking thread status
		snooze(300000.0);

		// if the thread is still around then bring up status window
		if (get_thread_info(gtid, &info) == B_NO_ERROR) {
			wind = new TStatusWindow("Creating gzip file... this may take a while!");
			wait_for_thread(gtid, &result);
			wind->Close();
		}
	} else {
		alert = new BAlert("", "Sorry, could not find the gzip tool to make an archive.", "Cancel");
		alert->Go();
	}



exit:
	for (i = 0; i < count; i++)
		free(t_argv[i + TAR_PARAMS]);
	free(t_argv);
	free(g_argv);
	
}

//-----------------------------------------------------------------	
// 
// TStatusView::TStatusView
// 
//-----------------------------------------------------------------	

TStatusView::TStatusView(BRect r,char *winstr)
		    :BView(r, "", B_FOLLOW_NONE, B_WILL_DRAW | B_PULSE_NEEDED)
{
	drawstring = winstr;
}

//-----------------------------------------------------------------	
// 
// TStatusView::Draw
// 
//-----------------------------------------------------------------	
void TStatusView::Draw(BRect)
{
	SetFontName("Erich");
	SetFontSize(9);
	MovePenTo(10, 20);
	DrawString(drawstring);
}

// ---------------------------------------------------------------

//-----------------------------------------------------------------	
// 
// TStatusWindow::TStatusWindow
// 
//-----------------------------------------------------------------	

TStatusWindow::TStatusWindow(char *winstr)
			  :BWindow(BRect(100, 120, 300, 160), "TarNgzip b1.03", B_TITLED_WINDOW,
					   B_NOT_CLOSABLE | B_NOT_RESIZABLE)
{
	BRect	status_wind_rect(100, 120, 300, 160);

	status_wind_rect.OffsetTo(B_ORIGIN);
	AddChild(new TStatusView(status_wind_rect, winstr));
	Show();
}

//-------------------------------------------------------------
//
// DbugMsg -- Quick and dirty alert box
// 		
//-------------------------------------------------------------
static void 
DbugMsg(char *psMsg)
{
	long 				lResult;
	BAlert 			*ba_Alert;
	
	ba_Alert = new BAlert("debug message", psMsg,
						"Cancel", NULL, NULL,
						B_WIDTH_FROM_WIDEST, B_WARNING_ALERT);
	ba_Alert->SetShortcut(0, B_ESCAPE);
	lResult = ba_Alert->Go();

}
